#ifndef POKEPLATINUM_UNK_0200762C_H
#define POKEPLATINUM_UNK_0200762C_H

#include "struct_decls/struct_02007768_decl.h"
#include "struct_defs/struct_02007C10.h"
#include "functypes/funcptr_02007C34.h"
#include "struct_decls/struct_02007C7C_decl.h"
#include "struct_defs/struct_02008900.h"
#include "struct_defs/struct_02008A90.h"

void * sub_0200762C(int param0);
void sub_02007768(UnkStruct_02007768 * param0);
void sub_02007B6C(UnkStruct_02007768 * param0);
void sub_02007B98(UnkStruct_02007C7C * param0, int param1);
void sub_02007C10(UnkStruct_02007C7C * param0, UnkStruct_02007C10 * param1);
BOOL sub_02007C24(UnkStruct_02007C7C * param0);
UnkStruct_02007C7C * sub_02007C34(UnkStruct_02007768 * param0, UnkStruct_02008A90 * param1, int param2, int param3, int param4, int param5, UnkStruct_02007C10 * param6, UnkFuncPtr_02007C34 * param7);
UnkStruct_02007C7C * sub_02007C7C(UnkStruct_02007768 * param0, UnkStruct_02008A90 * param1, int param2, int param3, int param4, int param5, int param6, UnkStruct_02007C10 * param7, UnkFuncPtr_02007C34 * param8);
void sub_02007DC8(UnkStruct_02007C7C * param0);
void sub_02007DD4(UnkStruct_02007768 * param0);
void sub_02007DEC(UnkStruct_02007C7C * param0, int param1, int param2);
int sub_020080C0(UnkStruct_02007C7C * param0, int param1);
void sub_02008274(UnkStruct_02007C7C * param0, int param1, int param2);
void sub_020086D4(UnkStruct_02007C7C * param0, int param1, int param2, int param3, int param4);
void sub_020086FC(UnkStruct_02007C7C * param0, int param1, int param2, int param3, int param4);
void sub_0200872C(UnkStruct_02007768 * param0, int param1, int param2, int param3, int param4);
void sub_02008780(UnkStruct_02007C7C * param0);
BOOL sub_020087B4(UnkStruct_02007C7C * param0);
void sub_020087C8(UnkStruct_02007C7C * param0, int param1);
void sub_020088E0(UnkStruct_02008900 * param0, const UnkStruct_02007C10 * param1);
int sub_02008900(UnkStruct_02008900 * param0);
void sub_020089A0(UnkStruct_02007C7C * param0);
void sub_020089B0(UnkStruct_02007C7C * param0);
void sub_02008A0C(UnkStruct_02007C7C * param0);
void sub_02008A78(UnkStruct_02007768 * param0, u32 param1, u32 param2);
void sub_02008A84(UnkStruct_02007768 * param0, u32 param1, u32 param2);
UnkStruct_02008A90 * sub_02008A90(UnkStruct_02007C7C * param0);
void sub_02008A94(UnkStruct_02007768 * param0);
void sub_02008B2C(UnkStruct_02007768 * param0, int param1);
BOOL sub_02008B38(UnkStruct_02007C7C * param0);
void sub_02008B54(UnkStruct_02007768 * param0, u32 param1);
void sub_02008B60(UnkStruct_02007768 * param0, u32 param1);
void sub_020091D8(u8 * param0, u32 param1, BOOL param2);
void sub_02009348(u8 * param0);
void sub_02009370(u8 * param0);
void sub_020093A0(u8 * param0, int param1);

#endif // POKEPLATINUM_UNK_0200762C_H
